
/*
 * $Id: Opt.cc,v 1.1.1.2 1999/02/19 07:43:01 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <ctype.h>
#include <iomanip.h>

#include "Assert.h"
#include "Opt.h"
#include "LogComment.h"
#include "gadgets.h"
#include "globals.h"

/* OptGrp */

void OptGrp::share(OptGrp *other) {
	Assert(other);
	for (int i = 0; i < theCount; ++i)
		other->append(item(i));
}

/* Opt */

Opt::Opt(OptGrp *aGrp, const char *aName, const char *aDescr):
	theGrp(aGrp), theCmdLine(0), theName(aName), theDescr(aDescr) {

	if (char *h = strchr(theName, ' ')) {
		theType = h+1;
		*h = '\0';
	} else {
		theType = "";
	}

	if (theGrp)
		theGrp->append(this); // self registration
}

Opt::~Opt() {
}


/* CmdLine utils */

static
bool namesEq(const char *name, const char *canonic) {
	if (!name || !canonic)
		return false;
	
	// strip leading '-'s
	while (*name == '-') name++;
	while (*canonic == '-') canonic++;

	if (!*name)
		return false;

	//clog << here << "comapring: " << name << " ? " << canonic << endl;

	return strcasecmp(name, canonic) == 0;
}

// returns val if it looks like a value rather than an option name
const char *peekVal(const char *val) {
	if (!val || val[0] != '-')
		return val;
	if (isdigit(val[1])) // negative integer's are valid values
		return val;
	return 0;
}


/* CmdLine */

CmdLine::CmdLine(const Array<OptGrp*> &groups): thePrgName("?") {

	// absorb all options
	for (int g = 0; g < groups.count(); ++g) {
		for (int i = 0; i < groups[g]->count(); ++i) {
			Opt *opt = groups[g]->item(i);
			opt->cmdLine(this);
			theOpts.append(opt);
		}
	}
}

CmdLine::~CmdLine() {
	while (theArgs.count()) delete[] theArgs.pop();
}

bool CmdLine::parse(int argc, char *argv[]) {
	for (int i = 0; i < argc; ++i)
		theArgs << StrDup(argv[i]);

	if (theArgs.count())
		thePrgName = ThePrgName = theArgs[0];

	// XXX: kludge; we should probably use a no-param option or something
	if (theArgs.count() <= 1) {
		usage(cerr);
		return false;
	}

	for (int i = 1; i < theArgs.count(); ++i) {
		const char *val = peekVal(theArgs[i+1]);

		if (*theArgs[i] != '-') {
			cerr << thePrgName << ": expected --option, found '" << theArgs[i] << "'" << endl;
			return false;
		}
		if (!parse(theArgs[i], val))
			return false;
		if (val) // skip option's value if any
			i++;
	}

	return true;
}

bool CmdLine::parse(const char *name, const char *val) {
	for (int i = 0; i < theOpts.count(); ++i) {
		if (namesEq(name, theOpts[i]->name()))
			return theOpts[i]->parse(name, val);
	}
	cerr << thePrgName << ": unknown option: `"
		<< name << ' ' << (val ? val : "") << "'"
		<< endl;
	return false;
}

void CmdLine::usage(ostream &os) const {
	os << "usage: "	<< thePrgName
		<< " [--option] ..." << endl;
	os << "options:" << endl;
	for (int i = 0; i < theOpts.count(); ++i) {
		const Opt *o = theOpts[i];
		const int plen = strlen(o->name()) + strlen(o->type()) + 1;
		os
			<< "  " << o->name() << ' ' << o->type()
			<< setw(20-plen) << ""
			<< "  " << o->descr()
			<< endl;
	}
}

void CmdLine::report() const {
	Comment << "Command:";
	for (int i = 0; i < theArgs.count(); ++i)
		Comment << ' ' << theArgs[i];
	Comment << endc;

	Comment(1) << "Configuration:";
	for (int i = 0; i < theOpts.count(); ++i) {
		if (!theOpts[i]->visible())
			continue;
		Comment << endl;
		const int plen = strlen(theOpts[i]->name()) + 1;
		Comment << "\t" << theOpts[i]->name() << ':' << setw(20-plen) << "";
		theOpts[i]->report();
	}
	Comment << endc;
}
