
/*
 * $Id: Socket.cc,v 1.1.1.7 1999/02/22 06:19:52 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h> /* for TCP_NODELAY */
#include <arpa/inet.h>
#include <netdb.h>

#include "Socket.h"
#include "Assert.h"

bool Socket::IsEnabled = true;
int  Socket::TheMaxLevel = -1;
int  Socket::TheLevel = 0;

bool Socket::create() {
	return create(PF_INET, SOCK_STREAM, 0);
}

bool Socket::create(int domain, int type, int protocol) {

	if (TheMaxLevel >= 0 && TheLevel >= TheMaxLevel) {
		errno = EMFILE;
		return false;
	}
	
	theFD = ::socket(domain, type, protocol);
	if (theFD >= 0)
		TheLevel++;
	return theFD >= 0;
}

bool Socket::bind(int port) {
	static sockaddr_in S;

	S.sin_family = AF_INET;
	S.sin_port = htons(port);
	S.sin_addr.s_addr = htonl(INADDR_ANY);

	return !IsEnabled || 
		::bind(theFD, (sockaddr *) &S, sizeof(S)) == 0;
}

bool Socket::connect(const NetAddr &addr) {
	static sockaddr_in s;

	s.sin_family = AF_INET;
	s.sin_addr = addr.addrN();
	s.sin_port = htons(addr.port());

	if (IsEnabled && ::connect(theFD, (sockaddr *) &s, sizeof(s)) < 0)
		return errno == EINPROGRESS;
	return true;
}

bool Socket::listen(int port) {

	if (!bind(port))
		return false;

	return !IsEnabled ||
		::listen(theFD, FD_SETSIZE) == 0;
}

Socket Socket::accept(sockaddr *addr_p, int *addr_len_p) {

	if (TheMaxLevel >= 0 && TheLevel >= TheMaxLevel) {
		errno = EMFILE;
		return -1;
	}

	sockaddr addr;
	int addr_len = sizeof(addr);
	if (!addr_p) {
		addr_p = &addr;
		addr_len_p = &addr_len;
	} else
		Assert(addr_len_p);

	const int fd = IsEnabled ?
		::accept(theFD, addr_p, addr_len_p) :
		(errno = EWOULDBLOCK, -1);

	if (fd >= 0)
		TheLevel++;

	return fd;
}

Size Socket::read(void *buf, Size sz) {
	return IsEnabled && sz > 0 ? (Size)::read(theFD, buf, sz) : (Size)0;
}

Size Socket::write(const void *buf, Size sz) {
	return IsEnabled && sz > 0 ? (Size)::write(theFD, buf, sz) : sz;
}

bool Socket::close() {
	if (theFD >= 0) {
		const int res = ::close(theFD);
		theFD = -1;
		TheLevel--;
		return res == 0;
	}
	errno = EBADF;
	return false;
}

bool Socket::sendBuf(Size sz) {
#ifdef SO_SNDBUF
	return setOpt(SO_SNDBUF, (const void*)&sz, sizeof(sz));
#else
	sz = 0; // make compiler happy
	return false;
	#warning SO_SNDBUF is not supported; no big deal unless you use it
#endif
}

bool Socket::recvBuf(Size sz) {
#ifdef SO_RCVBUF
	return setOpt(SO_RCVBUF, (const void*)&sz, sizeof(sz));
#else
	sz = 0; // make compiler happy
	return false;
	#warning SO_RCVBUF is not supported; no big deal unless you use it
#endif
}

int Socket::flags() const {

	const int flags = fcntl(theFD, F_GETFL);
	return flags;
}

/* local port */
int Socket::lport() const {
	sockaddr_in S;
	int len = sizeof(S);

	if (getsockname(theFD, (sockaddr*)&S, &len) == 0)
		return ntohs(S.sin_port);
	else
		return -1;
}

/* remote (peer) port */
int Socket::rport() const {
	sockaddr_in S;
	int len = sizeof(S);

	if (getpeername(theFD, (sockaddr*)&S, &len) == 0)
		return ntohs(S.sin_port);
	else
		return -1;
}

Time Socket::rcvTimeout() const {
#ifdef SO_RCVTIMEO
	Time t;
	int len = sizeof(t);
	if (getOpt(SO_RCVTIMEO, (void*)&t, &len))
		return t;
	else
		return Time();
#else
	#warning SO_RCVTIMEO is not supported; no big deal unless you use it
	return Time();
#endif
}

Size Socket::sendBuf() const {
#ifdef SO_SNDBUF
	Size sz;
	int len = sizeof(sz);
	if (getOpt(SO_SNDBUF, (void*)&sz, &len))
		return sz;
	else
		return Size();
#else
	return Size();
#endif
}

Size Socket::recvBuf() const {
#ifdef SO_RCVBUF
	Size sz;
	int len = sizeof(sz);
	if (getOpt(SO_RCVBUF, (void*)&sz, &len))
		return sz;
	else
		return Size();
#else
	return Size();
#endif
}

int Socket::errNo() const {
	int e;
	int len = sizeof(e);

	// XXX: this may not work on Solaris
	if (getOpt(SO_ERROR, (void*)&e, &len))
		return e;
	else
		return -1;
}

bool Socket::peerAddr(sockaddr *S, int &len) const {
	return getpeername(theFD, S, &len) == 0;
}

NetAddr Socket::peerAddr() const {
	sockaddr_in s;
	int len = sizeof(s);
	if (peerAddr((sockaddr *) &s, len))
		return NetAddr(inet_ntoa(s.sin_addr), ntohs(s.sin_port));
	else
		return NetAddr();
}

bool Socket::reuseAddr(bool set) {
#ifdef SO_REUSEADDR
	return setOpt(SO_REUSEADDR, set);
#else
	return set = false; // make compiler happy
	#warning SO_REUSEADDR is not supported; usually no big deal
#endif
}

bool Socket::linger(int delay) {
#ifdef SO_LINGER
	static struct linger l = { delay > 0, delay };
	return setOpt(SO_LINGER, &l, sizeof(l));
#else
	delay = 0; // make compiler happy
	return false;
	#warning SO_LINGER is not supported; no big deal unless you use it
#endif
}

bool Socket::blocking(bool set) {

	int flags = this->flags();

	if (set)
		flags &= ~O_NONBLOCK;
	else
		flags |=  O_NONBLOCK;

	return fcntl(theFD, F_SETFL, flags) != -1;
}

bool Socket::reusePort(bool set) {
#ifdef SO_REUSEPORT
	return setOpt(SO_REUSEPORT, set);
#else
	return set = false; // make compiler happy
	#warning SO_REUSEPORT is not supported; usually no big deal
#endif
}

bool Socket::tcpDelay(bool set) {
#ifdef TCP_NODELAY
	int optVal = (int)!set;
	// must cast to char* on Solaris
	return ::setsockopt(theFD, IPPROTO_TCP, TCP_NODELAY, (const char*)&optVal, sizeof(optVal)) == 0;
#else
	return set = false; // make compiler happy
	#warning TCP_NODELAY is not supported; no big deal unless you use it
#endif
}

bool Socket::isBlocking() const {
	return flags() & O_NONBLOCK == 0;
}


bool Socket::setOpt(int optName, bool set) {
	int optVal = (int)set;
	return setOpt(optName, &optVal, sizeof(optVal));
}

bool Socket::setOpt(int optName, const void *optVal, int optLen) {
	// must cast to char* on Solaris
	return ::setsockopt(theFD, SOL_SOCKET, optName, (const char*)optVal, optLen) == 0;
}

bool Socket::getOpt(int optName, void *optVal, int *optLen) const {
	// must cast to char* on Solaris
	return ::getsockopt(theFD, SOL_SOCKET, optName, (char*)optVal, optLen) == 0;
}
